"use strict";

var _global = require("./global");

var _parser = _interopRequireDefault(require("./parser"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function checkCondition(obj, relation, parent, root, self, lang) {
  if (_global.JS._.isUndefined(lang)) lang = _global.JE.i18nLang;
  let res = [];

  try {
    let keys = Object.keys(obj);

    for (let i = 0; i < keys.length; i++) {
      let key = keys[i];
      let value = obj[key];
      let result = true;

      if (key === '$or' || key === '$and' || key === '$not') {
        result = checkCondition(value, key.replace('$', ''), parent, root, self);
      } else {
          let comparision = _parser.default.parseComparision(key, value);

          result = compare(comparision, parent, root, self);
        }

      if (relation === 'and') {
        if (result === false) return false;else res.push(true);
      } else if (relation === 'or') {
        if (result === true) return true;else res.push(false);
      } else res.push(result);
    }

    if (relation === 'and') return true;else if (relation === 'or') return false;else if (relation === 'not') {
      let summary = true;

      _global.JS._.forEach(res, item => {
        summary = summary && item;
      });

      return !summary;
    } else _global.JS.throwError('ConditionDefError', null, null, [['zh-cn', '运算条件定义有误'], ['en-us', 'Invalid condition definition']], lang);
  } catch (err) {
    _global.JS.throwError('ConditionCalError', err, null, [['zh-cn', '条件运算出错'], ['en-us', 'Error occurred in condation calculation']], lang);
  }
}

module.exports.checkCondition = checkCondition;

function compare(comparision, parent, root, self, lang) {
  if (_global.JS._.isUndefined(lang)) lang = _global.JE.i18nLang;

  try {
    let left = comparision.left,
        right = comparision.right,
        operator = comparision.operator;
    if (_global.JS._.isString(left)) left = tag2value(left, parent, root, self);

    if (_global.JS._.isString(right)) {
      if (self === null) right = tag2value(right, parent, root, self);else right = tag2value(right, parent, root, null);
    }

    if (left === '{NotExist}' && operator !== 'exist' && operator !== 'notExist') {
      _global.JS.throwError('TagDefError', null, null, [['zh-cn', `标签 '${comparision.left}' 路径中有不存在的属性`], ['en-us', `Some property does not exists in path '${comparision.left}'`]], lang);
    }

    switch (operator) {
      case 'eq':
        return left == right;

      case 'ne':
        return left != right;

      case 'gte':
        return left >= right;

      case 'gt':
        return left > right;

      case 'lte':
        return left <= right;

      case 'lt':
        return left < right;

      case 'in':
        return right.indexOf(left) >= 0;

      case 'notIn':
        return right.indexOf(left) < 0;

      case 'contains':
        return left.indexOf(right) >= 0;

      case 'notContain':
        return left.indexOf(right) < 0;

      case 'startsWith':
        return left.indexOf(right) === 0;

      case 'notStartWith':
        return left.indexOf(right) !== 0;

      case 'endsWith':
        return left.indexOf(right) === left.length - right.length;

      case 'notEndWith':
        return left.indexOf(right) !== left.length - right.length;

      case 'matches':
        return left.search(right) >= 0;

      case 'notMatch':
        return left.search(right) < 0;

      case 'exists':
        return left != '{NotExist}' == right;

      case 'notExist':
        return left == '{NotExist}' == right;

      case 'isNull':
        return _global.JS._.isNull(left) == right;

      case 'isNotNull':
        return _global.JS._.isNull(left) != right;

      case 'isUndefined':
        return _global.JS._.isUndefined(left) == right;

      case 'isNotUndefined':
        return _global.JS._.isUndefined(left) != right;

      case 'isEmpty':
        return left == '' == right;

      case 'isNotEmpty':
        return left != '' == right;

      default:
        _global.JS.throwError('OperatorDefError', null, null, [['zh-cn', `运算符 '${operator}' 不存在`], ['en-us', `Invalid operator '${operator}'`]], lang);

    }
  } catch (err) {
    _global.JS.throwError('CompareError', err, null, [['zh-cn', `比较运算失败`], ['en-us', `Failded to execute comparision`]], lang);
  }
}

module.exports.compare = compare;

function tag2value(tag, parent, root, self, lang) {
  if (_global.JS._.isUndefined(lang)) lang = _global.JE.i18nLang;

  try {
    let target = null;

    if (tag.indexOf('/') === 0) {
      let objName = tag.split('/')[1].split('.')[0];

      if (Object.keys(root).indexOf(objName) < 0) {
        _global.JS.throwError('TagRefNotFilled', null, {
          needRef: objName
        }, [['zh-cn', `引用对象 /${objName} 尚未执行查询`], ['en-us', `Reference /${objName} has not been filled by data`]], lang);
      }

      target = root[objName].data;
    } else if (tag.indexOf('$') === 0) {
      target = parent;
    } else {
      if (self === null) return tag;else return self[tag];
    }

    let slices = tag.split('.$.');

    if (slices.length === 1) {
      let pieces = tag.split('.');
      let val = target;
      if (pieces.length === 1) return val;

      for (let i = 1; i < pieces.length; i++) {
        if (Object.prototype.toString.call(val) !== '[object Object]') {
          _global.JS.throwError('TagDefError', null, null, [['zh-cn', `'${pieces[i - 1]}' 不是 Object 类型`], ['en-us', `'${pieces[i - 1]}' is not an Object`]], lang);
        }

        if (Object.keys(val).indexOf(pieces[i]) < 0) {
          if (i < pieces.length - 1) {
            _global.JS.throwError('TagDefError', null, null, [['zh-cn', `不存在 '${pieces[i]}' 属性`], ['en-us', `'${pieces[i]}' property does not exist`]], lang);
          } else return '{NotExist}';
        }

        val = val[pieces[i]];
      }

      return val;
    } else if (slices.length === 2) {
      let pieces = slices[0].split('.');
      let val = target;
      if (pieces.length === 1) return val;

      for (let i = 1; i < pieces.length; i++) {
        if (Object.prototype.toString.call(val) !== '[object Object]') {
          _global.JS.throwError('TagDefError', null, null, [['zh-cn', `'${pieces[i - 1]}' 不是 Object 类型`], ['en-us', `'${pieces[i - 1]}' is not an Object`]], lang);
        }

        if (Object.keys(val).indexOf(pieces[i]) < 0) {
          if (i < pieces.length - 1) {
            _global.JS.throwError('TagDefError', null, null, [['zh-cn', `不存在 '${pieces[i]}' 属性`], ['en-us', `'${pieces[i]}' property does not exist`]], lang);
          } else return '{NotExist}';
        }

        val = val[pieces[i]];
      }

      if (slices[1].split('.').length > 1) {
        _global.JS.throwError('TagDefError', null, null, [['zh-cn', `无法取得一个数组的级联子属性`], ['en-us', `You can't aquire a cascaded property of an array`]], lang);
      }

      if (!_global.JS._.isArray(val)) {
        _global.JS.throwError('TagDefError', null, null, [['zh-cn', `属性 '${pieces[pieces.length - 1]}' 不是 Array 类型，无法进行子属性抽取`], ['en-us', `You can't pick element properties from a non-array Object '${pieces[pieces.length - 1]}'`]], lang);
      }

      return _global.JS._.uniq(_global.JS._.map(val, slices[1]));
    } else _global.JS.throwError('TagDefError', null, null, [['zh-cn', `不支持多层 '.$.' 查询`], ['en-us', `Multiple '.$.' query is not supported`]], lang);
  } catch (err) {
    _global.JS.throwError('Tag2ValueError', err, null, [['zh-cn', `标签转为值失败`], ['en-us', `Failed to convert tag '${tag}' to value`]], lang);
  }
}

module.exports.tag2value = tag2value;

function getValue(list, obj, lang) {
  if (_global.JS._.isUndefined(lang)) lang = _global.JE.i18nLang;

  try {
    let result = null;

    switch (obj.operator) {
      case 'count':
        {
          if (obj.fields === '*') return list.length;
          let slices = obj.fields.split(',');
          if (slices.length > 1) _global.JS.throwError('FieldsDefError', null, null, [['zh-cn', `'${obj.operator}' 运算只接受一个字段`], ['en-us', `Calculation '${obj.operator}' accepts one filed only`]], lang);
          result = 0;

          _global.JS._.forEach(list, item => {
            if (Object.keys(item).indexOf(obj.fields) >= 0 && !_global.JS._.isNull(item[obj.fields])) result++;
          });

          return result;
        }

      case 'sum':
        {
          if (list.length === 0) return null;
          let slices = obj.fields.split(',');
          if (slices.length > 1) _global.JS.throwError('FieldsDefError', null, null, [['zh-cn', `'${obj.operator}' 运算只接受一个字段`], ['en-us', `Calculation '${obj.operator}' accepts one filed only`]], lang);
          return _global.JS._.sumBy(list, obj.fields);
        }

      case 'avg':
        {
          if (list.length === 0) return null;
          let slices = obj.fields.split(',');
          if (slices.length > 1) _global.JS.throwError('FieldsDefError', null, null, [['zh-cn', `'${obj.operator}' 运算只接受一个字段`], ['en-us', `Calculation '${obj.operator}' accepts one filed only`]], lang);
          return _global.JS._.meanBy(list, obj.fields);
        }

      case 'max':
        {
          if (list.length === 0) return null;
          let slices = obj.fields.split(',');
          if (slices.length > 1) _global.JS.throwError('FieldsDefError', null, null, [['zh-cn', `'${obj.operator}' 运算只接受一个字段`], ['en-us', `Calculation '${obj.operator}' accepts one filed only`]], lang);
          return _global.JS._.maxBy(list, obj.fields)[obj.fields];
        }

      case 'min':
        {
          if (list.length === 0) return null;
          let slices = obj.fields.split(',');
          if (slices.length > 1) _global.JS.throwError('FieldsDefError', null, null, [['zh-cn', `'${obj.operator}' 运算只接受一个字段`], ['en-us', `Calculation '${obj.operator}' accepts one filed only`]], lang);
          return _global.JS._.minBy(list, obj.fields)[obj.fields];
        }

      case 'first':
        {
          if (list.length === 0) return null;
          let slices = obj.fields.split(',');
          if (slices.length > 1) _global.JS.throwError('FieldsDefError', null, null, [['zh-cn', `'${obj.operator}' 运算只接受一个字段`], ['en-us', `Calculation '${obj.operator}' accepts one filed only`]], lang);else {
            let item = list[0];
            if (Object.keys(item).indexOf(obj.fields) < 0) _global.JS.throwError('FieldsDefError', null, null, [['zh-cn', `'${obj.fields}' 字段不存在于第一条记录`], ['en-us', `Field '${obj.fields}' doesn't exist in first row of records`]], lang);else return item[obj.fields];
          }
        }

      case 'pick':
        {
          if (list.length === 0) return [];
          let slices = obj.fields.split(',');
          if (slices.length > 1) _global.JS.throwError('FieldsDefError', null, null, [['zh-cn', `'${obj.operator}' 运算只接受一个字段`], ['en-us', `Calculation '${obj.operator}' accepts one filed only`]], lang);
          return _global.JS._.uniq(_global.JS._.map(list, obj.fields));
        }

      case 'clone':
        {
          if (list.length === 0) return null;
          let fields = [];
          let slices = obj.fields.split(',');

          _global.JS._.forEach(slices, item => {
            if (item !== '') fields.push(item);
          });

          fields = _global.JS._.uniq(fields);
          if (fields.indexOf('*') >= 0) fields = '*';
          result = [];

          _global.JS._.forEach(list, item => {
            let temp = {};

            if (fields === '*') {
              let keys = Object.keys(item);

              _global.JS._.forEach(keys, key => {
                temp[key] = _global.JS._.cloneDeep(item[key]);
              });
            } else {
              _global.JS._.forEach(fields, field => {
                temp[field] = _global.JS._.cloneDeep(item[field]);
              });
            }

            result.push(temp);
          });

          if (result.length === 0) result = null;
          return result;
        }

      default:
        {
          _global.JS.throwError('OperatorDefError', null, null, [['zh-cn', `无效的运算符 '${obj.operator}'`], ['en-us', `Invalid operator '${obj.operator}'`]], lang);
        }
    }
  } catch (err) {
    _global.JS.throwError('ValuesCalError', err, null, [['zh-cn', `查询后取值失败`], ['en-us', `Getting value failed after query`]], lang);
  }
}

module.exports.getValue = getValue;