"use strict";

var _global = require("./global");

function parseFields(fields, lang) {
  if (_global.JS._.isUndefined(lang)) lang = _global.JE.i18nLang;
  if (_global.JS._.isUndefined(fields)) return {
    raw: '*',
    values: [],
    cascaded: []
  };
  let values = [];
  let rawFields = [];
  let cascadedFields = [];

  try {
    let list = [];

    if (_global.JS._.isString(fields)) {
      if (fields === '*') list.push('*');else {
        let slices = fields.split(',');
        if (slices.length === 1) list.push(slices[0]);else {
          for (let i = 0; i < slices.length; i++) {
            if (slices[i] !== '') list.push(slices[i]);
          }
        }
      }
    } else list = fields;

    let hasStar = false;

    _global.JS._.forEach(list, item => {
      if (_global.JS._.isString(item)) {
        if (item === '*') {
          rawFields.push('*');
          hasStar = true;
        } else {
          if (hasStar) _global.JS.throwError('FieldsDefError', null, null, [['zh-cn', `字段 '${item}' 定义有误，前面已经有 * 就不能再定义其它字段名`], ['en-us', `Invalid field '${item}' definition，no more fields can be defined after a '*'`]], lang);
          let slices = item.split('=>');
          if (slices.length === 1) rawFields.push(item);else if (slices.length === 2) {
            if (slices[0] === '' || slices[1] === '') _global.JS.throwError('FieldsDefError', null, null, [['zh-cn', `字段 '${item}' 定义有误，'=>' 两侧都不能是空字符串`], ['en-us', `Invalid field '${item}' definition，empty String is not allowed in both left and right sides around '=>'`]], lang);
            let pieces = slices[0].split('#');

            if (pieces.length === 1) {
              let temp = {};
              temp[slices[1]] = slices[0];
              rawFields.push(temp);
            } else if (pieces.length === 2) {
              if (pieces[0] === '' || pieces[1] === '') _global.JS.throwError('FieldsDefError', null, null, [['zh-cn', `字段 '${item}' 定义有误，'#' 两侧都不能是空字符串`], ['en-us', `Invalid field '${item}' definition，empty String is not allowed in both left and right sides around '#'`]], lang);
              let temp = {
                name: slices[1],
                operator: pieces[0],
                fields: pieces[1]
              };
              values.push(temp);
            } else _global.JS.throwError('FieldsDefError', null, null, [['zh-cn', `字段 '${item}' 定义有误，有多于1个 '#' 符号`], ['en-us', `Invalid field '${item}' definition，having more than one '#' is not allowed`]], lang);
          } else _global.JS.throwError('FieldsDefError', null, null, [['zh-cn', `字段 '${item}' 定义有误，有多于1个 '=>' 符号`], ['en-us', `Invalid field '${item}' definition，having more than one '=>' is not allowed`]], lang);
        }
      } else if (Object.prototype.toString.call(item) === '[object Object]') {
        cascadedFields.push(item);
      } else _global.JS.throwError('FieldsDefError', null, null, [['zh-cn', `'fields' 数组元素必须是 String 或者 Object`], ['en-us', `Each element in 'fields' must be a String or an Object`]], lang);
    });

    if (rawFields.indexOf('*') >= 0 && rawFields.length > 1) _global.JS.throwError('FieldsDefError', null, null, [['zh-cn', `请检查 'fields' 定义，* 不能与其它字段定义同时出现`], ['en-us', `Please check the 'fields' property, once you have a '*', other fields are not allowed to exist`]], lang);
    if (rawFields.indexOf('*') >= 0 && rawFields.length === 1) rawFields = '*';
    return {
      raw: rawFields,
      values: values,
      cascaded: cascadedFields
    };
  } catch (err) {
    _global.JS.throwError('FieldsPaserError', err, null, [['zh-cn', `解析 fields 出错`], ['en-us', `Error occurred while parsing 'fields' property`]], lang);
  }
}

module.exports.parseFields = parseFields;

function parseDataString(input, lang) {
  if (_global.JS._.isUndefined(lang)) lang = _global.JE.i18nLang;

  try {
    let slices = input.split(':');
    if (input.indexOf(' ') >= 0) _global.JS.throwError('DataDefError', null, null, [['zh-cn', `表达式 '${input}' 不能含有空格`], ['en-us', `Spaces are not allowed in expression '${input}'`]], lang);
    if (slices.length !== 2) _global.JS.throwError('DataDefError', null, null, [['zh-cn', `表达式 '${input}' 定义有误，必须且只能有一个 ':' 字符`], ['en-us', `Expression '${input}' is invalid, it must and can only have one ':'`]], lang);
    if (slices[0] === '' || slices[1] === '') _global.JS.throwError('DataDefError', null, null, [['zh-cn', `表达式 '${input}' 定义有误，':' 两侧都不能是空字符串`], ['en-us', `Expression '${input}' is invalid，empty String is not allowed in both left and right sides around ':'`]], lang);
    if (isNaN(parseFloat(slices[1]))) _global.JS.throwError('DataValueInvalidError', null, null, [['zh-cn', `表达式 '${input}' 定义有误，'${slices[1]}' 不是有效数字`], ['en-us', `Expression '${input}' is invalid, '${slices[1]}' is not a number`]], lang);
    if (parseFloat(slices[1]) <= 0) _global.JS.throwError('DataValueInvalidError', null, null, [['zh-cn', `表达式 '${input}' 定义有误，'${slices[1]}' 不能小于等于 0`], ['en-us', `Expression '${input}' is invalid, '${slices[1]}' must be a positive number or 0`]], lang);
    let temp = {};
    temp[slices[0]] = parseFloat(slices[1]);
    return temp;
  } catch (err) {
    _global.JS.throwError('DataPaserError', err, null, [['zh-cn', `解析 data 出错`], ['en-us', `Error occurred while parsing 'data' property`]], lang);
  }
}

module.exports.parseDataString = parseDataString;

function parseOrder(order, lang) {
  if (_global.JS._.isUndefined(lang)) lang = _global.JE.i18nLang;
  if (_global.JS._.isUndefined(order)) return [];

  try {
    if (_global.JS._.isString(order)) {
      let temp = [];
      let slices = order.split(',');

      for (let i = 0; i < slices.length; i++) {
        if (slices[i] !== '') temp.push(slices[i]);
      }

      order = temp;
    }

    if (_global.JS._.isArray(order)) {
      let result = [];

      for (let i = 0; i < order.length; i++) {
        if (!_global.JS._.isString(order[i])) _global.JS.throwError('OrderDefError', null, null, [['zh-cn', `下标为 ${i} 的元素不是 String 类型`], ['en-us', `The element with index ${i} is not a String`]], lang);
        let slices = order[i].split('#');

        if (slices.length === 1) {
          result.push({
            column: slices[0],
            order: 'asc'
          });
        } else if (slices.length === 2) {
          if (slices[0] === '' || slices[1] === '') _global.JS.throwError('OrderDefError', null, null, [['zh-cn', `'${order[i]}' 定义有误，'#' 符号两侧均不能为空字符串`], ['en-us', `'${order[i]}' is invalid，empty String is not allowed in both left and right sides around '#'`]], lang);
          result.push({
            column: slices[0],
            order: slices[1]
          });
        } else _global.JS.throwError('OrderDefError', null, null, [['zh-cn', `'${order[i]}' 定义有误，有多于1个 '#' 符号`], ['en-us', `'${order[i]}' is invalid，having more than one '#' is not allowed`]], lang);
      }

      return result;
    }
  } catch (err) {
    _global.JS.throwError('OrderPaserError', err, null, [['zh-cn', `解析 order 出错`], ['en-us', `Error occurred while parsing 'order' property`]], lang);
  }
}

module.exports.parseOrder = parseOrder;

function parseOffsetAndLimit(page, size, lang) {
  if (_global.JS._.isUndefined(lang)) lang = _global.JE.i18nLang;
  let result = {
    offset: 0,
    limit: 0
  };

  try {
    if (_global.JS._.isUndefined(page) && _global.JS._.isUndefined(size)) return result;

    if (!_global.JS._.isUndefined(size) && _global.JS._.isUndefined(page)) {
      result.limit = size;
      return result;
    }

    if (!_global.JS._.isUndefined(page) && _global.JS._.isUndefined(size)) {
      _global.JS.throwError('SizeDefError', null, null, [['zh-cn', `如果定义了 'page'，也必须定义 'size'`], ['en-us', `If you have defined a 'page', one 'size' definition is required too`]], lang);
    }

    if (!_global.JS._.isUndefined(size) && !_global.JS._.isUndefined(page)) {
      result.offset = (page - 1) * size;
      result.limit = size;
      return result;
    }
  } catch (err) {
    _global.JS.throwError('QueryPaserError', err, null, [['zh-cn', `解析 page 和 size 出错`], ['en-us', `Error occurred while parsing 'page' and 'size'`]], lang);
  }
}

module.exports.parseOffsetAndLimit = parseOffsetAndLimit;

function parseComparision(key, value, lang) {
  if (_global.JS._.isUndefined(lang)) lang = _global.JE.i18nLang;

  try {
    let comparision = {
      left: '',
      operator: 'eq',
      right: ''
    };
    let pieces = key.split('#');

    if (pieces.length === 1) {
      comparision.left = pieces[0];
      comparision.operator = 'eq';
      comparision.right = value;
    }

    if (pieces.length === 2) {
      if (pieces[0] === '' || pieces[1] === '') _global.JS.throwError('OpDefError', null, null, [['zh-cn', `'${key}' 定义有误，'#' 符号两侧均不能为空字符串`], ['en-us', `'${key}' is invalid，empty String is not allowed in both left and right sides around '#'`]], lang);
      comparision.left = pieces[0];
      comparision.operator = pieces[1];
      comparision.right = value;
    }

    if (pieces.length > 2) _global.JS.throwError('OpDefError', null, null, [['zh-cn', `'${key}' 定义有误，有多于1个 '#' 符号`], ['en-us', `'${key}' is invalid，having more than one '#' is not allowed`]], lang);
    return comparision;
  } catch (err) {
    _global.JS.throwError('ComparisionParserError', err, null, [['zh-cn', `单个比较运算条件解析失败`], ['en-us', `Error occurred while parsing single comparision`]], lang);
  }
}

module.exports.parseComparision = parseComparision;