"use strict";

var _global = require("./global");

var _parser = _interopRequireDefault(require("./parser"));

var _calculator = _interopRequireDefault(require("./calculator"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function getObjFromObj(data, rawFields, lang) {
  if (_global.JS._.isUndefined(lang)) lang = _global.JE.i18nLang;

  try {
    if (rawFields === '*') return data;
    let result = {};
    let keys = Object.keys(data);

    _global.JS._.forEach(rawFields, item => {
      if (_global.JS._.isString(item)) {
        if (keys.indexOf(item) < 0) _global.JS.throwError('FieldNotExistError', null, null, [['zh-cn', `原始数据中不存在 '${item}' 字段`], ['en-us', `Field '${item}' doesn't exist in raw data`]], lang);
        result[item] = data[item];
      }

      if (_global.JS._.isPlainObject(item)) {
        let key = Object.keys(item)[0];
        if (keys.indexOf(item[key]) < 0) _global.JS.throwError('FieldNotExistError', null, null, [['zh-cn', `原始数据中不存在 '${key}' 字段`], ['en-us', `Field '${key}' doesn't exist in raw data`]], lang);
        result[key] = data[item[key]];
      }
    });

    return result;
  } catch (err) {
    _global.JS.throwError('DealRefError', err, null, [['zh-cn', `单个对象属性筛选出错`], ['en-us', `Filtering fields from one Object failed`]], lang);
  }
}

module.exports.getObjFromObj = getObjFromObj;

function getObjFromList(data, query, rawFields, parent, root, lang) {
  if (_global.JS._.isUndefined(lang)) lang = _global.JE.i18nLang;

  try {
    let result = null;
    if (data.length === 0) return null;

    if (!_global.JS._.isUndefined(query)) {
      if (!_global.JS._.isUndefined(query.order)) {
        let orderFields = [];
        let orderDirections = [];

        let order = _parser.default.parseOrder(query.order);

        if (order.length > 0) {
          _global.JS._.forEach(order, item => {
            orderFields.push(item.column);
            orderDirections.push(item.order);
          });

          data = _global.JS._.orderBy(data, orderFields, orderDirections);
          result = data[0];
        }
      }

      if (!_global.JS._.isUndefined(query.where)) {
        let whereObj = {};

        if (Object.prototype.toString.call(query.where) === '[object Object]') {
          whereObj = query.where;
        }

        for (let i = 0; i < data.length; i++) {
          let item = data[i];

          if (_calculator.default.checkCondition(whereObj, 'and', parent, root, item) === true) {
            result = _global.JS._.cloneDeep(item);
            break;
          }
        }

        if (result == null) return null;
      }
    } else result = _global.JS._.cloneDeep(data[0]);

    if (result != null) result = getObjFromObj(result, rawFields);
    return result;
  } catch (err) {
    _global.JS.throwError('DealRefError', err, null, [['zh-cn', `多条记录中筛选目标出错`], ['en-us', `Selecting one target from multiple records failed`]], lang);
  }
}

module.exports.getObjFromList = getObjFromList;

function getListFromList(data, query, rawFields, parent, root, lang) {
  if (_global.JS._.isUndefined(lang)) lang = _global.JE.i18nLang;

  try {
    let result = [];
    if (data.length === 0) return null;

    if (!_global.JS._.isUndefined(query)) {
      if (!_global.JS._.isUndefined(query.where)) {
        let whereObj = {};

        if (Object.prototype.toString.call(query.where) === '[object Object]') {
          whereObj = query.where;
        }

        for (let i = 0; i < data.length; i++) {
          let item = data[i];

          if (_calculator.default.checkCondition(whereObj, 'and', parent, root, item) === true) {
            result.push(_global.JS._.cloneDeep(item));
          }
        }

        if (result == []) return null;
      } else result = _global.JS._.cloneDeep(data);

      if (!_global.JS._.isUndefined(query.order)) {
        let orderFields = [];
        let orderDirections = [];

        let order = _parser.default.parseOrder(query.order);

        if (order.length > 0) {
          _global.JS._.forEach(order, item => {
            orderFields.push(item.column);
            orderDirections.push(item.order);
          });

          result = _global.JS._.orderBy(result, orderFields, orderDirections);
        }
      }

      let temp = [];

      let pas = _parser.default.parseOffsetAndLimit(query.page, query.size);

      if (pas.limit > 0) {
        for (let i = pas.offset; i < pas.offset + pas.limit; i++) {
          if (i >= 0 && i < result.length) temp.push(result[i]);
        }
      } else {
        for (let i = pas.offset; i < result.length; i++) {
          temp.push(result[i]);
        }
      }

      result = temp;
    } else result = _global.JS._.cloneDeep(data);

    let ret = [];

    _global.JS._.forEach(result, item => {
      ret.push(getObjFromObj(item, rawFields));
    });

    return ret;
  } catch (err) {
    _global.JS.throwError('DealRefError', err, null, [['zh-cn', `多条记录中筛选多条记录出错`], ['en-us', `Selecting several records from multiple records failed`]], lang);
  }
}

module.exports.getListFromList = getListFromList;

function getValuesFromList(data, valuesFields, lang) {
  if (_global.JS._.isUndefined(lang)) lang = _global.JE.i18nLang;

  try {
    if (valuesFields.length === 0) _global.JS.throwError('CmdDefError', null, null, [['zh-cn', `'values' 查询类型至少要定义一个取值字段`], ['en-us', `A 'values' type query needs one value field defined at least`]], lang);
    let values = {};

    for (let i = 0; i < valuesFields.length; i++) {
      let item = valuesFields[i];
      values[item.name] = _calculator.default.getValue(data, item);
    }

    return values;
  } catch (err) {
    _global.JS.throwError('DealRefError', err, null, [['zh-cn', `取值查询出错`], ['en-us', `Error occurred in this 'values' type query`]], lang);
  }
}

module.exports.getValuesFromList = getValuesFromList;