"use strict";

var _global = require("./global");

const XSS = require('xss');

const options = {};
const xss = new XSS.FilterXSS(options);

function checkJson(json, lang) {
  if (_global.JS._.isUndefined(lang)) lang = _global.JE.i18nLang;
  if (Object.prototype.toString.call(json) !== '[object Object]') _global.JS.throwError('ParamTypeError', null, null, [['zh-cn', `传入的 JSON 必须是 Object 类型`], ['en-us', `The param must be an Object`]], lang);
  if (!_global.JS._.isUndefined(json.security) && Object.prototype.toString.call(json.security) !== '[object Object]') _global.JS.throwError('PropTypeError', null, null, [['zh-cn', `属性 'security' 必须是 Object 类型`], ['en-us', `The 'security' property must be an Object`]], lang);
  if (!_global.JS._.isUndefined(json.needLogs) && !_global.JS._.isBoolean(json.needLogs)) _global.JS.throwError('PropTypeError', null, null, [['zh-cn', `属性 'needLogs' 必须是 Boolean 类型`], ['en-us', `The 'needLogs' property must be a Boolean`]], lang);
  if (!_global.JS._.isUndefined(json.needTrace) && !_global.JS._.isBoolean(json.needTrace)) _global.JS.throwError('PropTypeError', null, null, [['zh-cn', `属性 'needTrace' 必须是 Boolean 类型`], ['en-us', `The 'needTrace' property must be a Boolean`]], lang);
  if (!_global.JS._.isUndefined(json.isTransaction) && !_global.JS._.isBoolean(json.isTransaction)) _global.JS.throwError('PropTypeError', null, null, [['zh-cn', `属性 'isTransaction' 必须是 Boolean 类型`], ['en-us', `The 'isTransaction' property must be a Boolean`]], lang);
  if (_global.JS._.isUndefined(json.commands)) _global.JS.throwError('PropMissingError', null, null, [['zh-cn', `属性 'commands' 必须要配置`], ['en-us', `The 'commands' property is required`]], lang);
  if (!_global.JS._.isArray(json.commands) && Object.prototype.toString.call(json.commands) !== '[object Object]') _global.JS.throwError('PropTypeError', null, null, [['zh-cn', `属性 'commands' 必须是 Array 或者 Object 类型`], ['en-us', `The 'commands' property must be an Array or an Object`]], lang);
  if (_global.JS._.isArray(json.commands) && json.commands.length === 0) _global.JS.throwError('PropEmptyError', null, null, [['zh-cn', `属性 'commands' 最少要包含一个指令`], ['en-us', `The 'commands' property requires at least one command`]], lang);
  return true;
}

module.exports.checkJson = checkJson;

function checkCommand(cmd, lang) {
  if (_global.JS._.isUndefined(lang)) lang = _global.JE.i18nLang;
  if (Object.prototype.toString.call(cmd) !== '[object Object]') _global.JS.throwError('ParamTypeError', null, null, [['zh-cn', `传入的 'command' 参数必须是 Object 类型`], ['en-us', `The param 'command' must be an Object`]], lang);
  let key = 'name';
  if (_global.JS._.isUndefined(cmd[key])) _global.JS.throwError('PropMissingError', null, null, [['zh-cn', `属性 '${key}' 必须要配置`], ['en-us', `The '${key}' property is required`]], lang);
  if (!_global.JS._.isString(cmd[key])) _global.JS.throwError('PropTypeError', null, null, [['zh-cn', `属性 '${key}' 必须是 String 类型`], ['en-us', `The '${key}' property must be a String`]], lang);
  if (cmd[key] === '') _global.JS.throwError('PropEmptyError', null, null, [['zh-cn', `属性 '${key}' 不能为空字符串`], ['en-us', `The '${key}' property can not be an empty String`]], lang);
  key = 'type';
  if (_global.JS._.isUndefined(cmd[key])) _global.JS.throwError('PropMissingError', null, null, [['zh-cn', `属性 '${key}' 必须要配置`], ['en-us', `The '${key}' property is required`]], lang);
  if (!_global.JS._.isString(cmd[key])) _global.JS.throwError('PropTypeError', null, null, [['zh-cn', `属性 '${key}' 必须是 String 类型`], ['en-us', `The '${key}' property must be a String`]], lang);
  let validCmds = ['entity', 'list', 'values', 'create', 'update', 'delete', 'increase', 'decrease', 'function'];
  if (validCmds.indexOf(cmd[key]) < 0) _global.JS.throwError('PropValueInvalidError', null, null, [['zh-cn', `属性 '${key}' 的值 '${cmd[key]}' 不合法`], ['en-us', `The value [${cmd[key]}] is invalid for property '${key}'`]], lang);
  key = 'target';
  if (_global.JS._.isUndefined(cmd[key])) _global.JS.throwError('PropMissingError', null, null, [['zh-cn', `属性 '${key}' 必须要配置`], ['en-us', `The '${key}' property is required`]], lang);
  if (!_global.JS._.isString(cmd[key])) _global.JS.throwError('PropTypeError', null, null, [['zh-cn', `属性 '${key}' 必须是 String 类型`], ['en-us', `The '${key}' property must be a String`]], lang);
  if (cmd[key] === '') _global.JS.throwError('PropEmptyError', null, null, [['zh-cn', `属性 '${key}' 不能为空字符串`], ['en-us', `The '${key}' property can not be an empty String`]], lang);
  key = 'onlyIf';
  if (!_global.JS._.isUndefined(cmd[key]) && Object.prototype.toString.call(cmd[key]) !== '[object Object]') _global.JS.throwError('PropTypeError', null, null, [['zh-cn', `属性 '${key}' 必须是 Object 类型`], ['en-us', `The '${key}' property must be an Object`]], lang);
  key = 'after';
  if (!_global.JS._.isUndefined(cmd[key]) && !_global.JS._.isArray(cmd[key]) && Object.prototype.toString.call(cmd[key]) !== '[object Object]') _global.JS.throwError('PropTypeError', null, null, [['zh-cn', `属性 '${key}' 必须是 Object 或 Array 类型`], ['en-us', `The '${key}' property must be an Object or an Array`]], lang);

  if (cmd.type === 'entity' || cmd.type === 'list' || cmd.type === 'values') {
    key = 'query';
    if (!_global.JS._.isUndefined(cmd[key]) && Object.prototype.toString.call(cmd[key]) !== '[object Object]') _global.JS.throwError('PropTypeError', null, null, [['zh-cn', `属性 '${key}' 必须是 Object 类型`], ['en-us', `The '${key}' property must be an Object`]], lang);
    key = 'fields';
    if (!_global.JS._.isUndefined(cmd[key]) && !_global.JS._.isArray(cmd[key]) && !_global.JS._.isString(cmd[key])) _global.JS.throwError('PropTypeError', null, null, [['zh-cn', `属性 '${key}' 必须是 Array 或者 String 类型`], ['en-us', `The '${key}' property must be an Array or a String`]], lang);
    if (_global.JS._.isArray(cmd[key]) && cmd[key].length === 0) _global.JS.throwError('PropEmptyError', null, null, [['zh-cn', `属性 '${key}' 最少要包含一个字段`], ['en-us', `The '${key}' property requires at least one field`]], lang);
    if (_global.JS._.isString(cmd[key]) && cmd[key] === '') _global.JS.throwError('PropEmptyError', null, null, [['zh-cn', `属性 '${key}' 不能为空字符串`], ['en-us', `The '${key}' property can not be an empty String`]], lang);
    key = 'data';
    if (!_global.JS._.isUndefined(cmd[key])) _global.JS.throwError('PropSpilthError', null, null, [['zh-cn', `属性 '${key}' 不应该出现在查询指令中`], ['en-us', `The '${key}' property should not exist in '${cmd.type}' type queries`]], lang);
  }

  if (cmd.type === 'create' || cmd.type === 'update' || cmd.type === 'delete' || cmd.type === 'increase' || cmd.type === 'decrease') {
    key = 'fields';
    if (!_global.JS._.isUndefined(cmd[key])) _global.JS.throwError('PropSpilthError', null, null, [['zh-cn', `属性 '${key}' 不应该出现在增删改指令中`], ['en-us', `The '${key}' property should not exist in '${cmd.type}' type commands`]], lang);
  }

  if (cmd.type === 'create' || cmd.type === 'update' || cmd.type === 'increase' || cmd.type === 'decrease' || cmd.type === 'function') {
    key = 'data';
    if (_global.JS._.isUndefined(cmd[key])) _global.JS.throwError('PropMissingError', null, null, [['zh-cn', `属性 '${key}' 必须要配置`], ['en-us', `The '${key}' property is required`]], lang);

    if (cmd.type === 'increase' || cmd.type === 'decrease') {
      if (Object.prototype.toString.call(cmd[key]) !== '[object Object]' && !_global.JS._.isString(cmd[key])) _global.JS.throwError('PropTypeError', null, null, [['zh-cn', `属性 '${key}' 必须是 Object 或 String 类型`], ['en-us', `The '${key}' property must be an Object or a String`]], lang);
    } else if (cmd.type === 'create') {
      if (!_global.JS._.isArray(cmd[key]) && Object.prototype.toString.call(cmd[key]) !== '[object Object]') _global.JS.throwError('PropTypeError', null, null, [['zh-cn', `属性 '${key}' 必须是 Object 或 Array 类型`], ['en-us', `The '${key}' property must be an Object or an Array`]], lang);
      if (_global.JS._.isArray(cmd[key]) && cmd[key].length === 0) _global.JS.throwError('PropEmptyError', null, null, [['zh-cn', `属性 '${key}' 最少要包含一个字段`], ['en-us', `The '${key}' property requires at least one field`]], lang);
    } else if (cmd.type === 'update') {
      if (Object.prototype.toString.call(cmd[key]) !== '[object Object]') _global.JS.throwError('PropTypeError', null, null, [['zh-cn', `属性 '${key}' 必须是 Object 类型`], ['en-us', `The '${key}' property must be an Object`]], lang);
    }
  }

  if (cmd.type === 'delete' || cmd.type === 'update' || cmd.type === 'increase' || cmd.type === 'decrease') {
    key = 'query';
    if (!_global.JS._.isUndefined(cmd[key]) && Object.prototype.toString.call(cmd[key]) !== '[object Object]') _global.JS.throwError('PropTypeError', null, null, [['zh-cn', `属性 '${key}' 必须是 Object 类型`], ['en-us', `The '${key}' property must be an Object`]], lang);
  }

  if (cmd.type === 'create') {
    key = 'query';
    if (!_global.JS._.isUndefined(cmd[key])) _global.JS.throwError('PropSpilthError', null, null, [['zh-cn', `属性 '${key}' 不应该出现在增删改指令中`], ['en-us', `The '${key}' property should not exist in '${cmd.type}' type commands`]], lang);
  }

  if (cmd.type === 'function') {
    key = 'query';
    if (!_global.JS._.isUndefined(cmd[key])) _global.JS.throwError('PropSpilthError', null, null, [['zh-cn', `属性 '${key}' 不应该出现在服务端函数指令中`], ['en-us', `The '${key}' property should not exist in '${cmd.type}' type commands`]], lang);
    key = 'fields';
    if (!_global.JS._.isUndefined(cmd[key])) _global.JS.throwError('PropSpilthError', null, null, [['zh-cn', `属性 '${key}' 不应该出现在服务端函数指令中`], ['en-us', `The '${key}' property should not exist in '${cmd.type}' type commands`]], lang);
  }

  return true;
}

module.exports.checkCommand = checkCommand;

function checkTopCommand(cmd, lang) {
  if (_global.JS._.isUndefined(lang)) lang = _global.JE.i18nLang;
  checkCommand(cmd, lang);
  let key = 'return';
  if (!_global.JS._.isUndefined(cmd[key]) && !_global.JS._.isBoolean(cmd[key])) _global.JS.throwError('PropTypeError', null, null, [['zh-cn', `属性 '${key}' 必须是 Boolean 类型`], ['en-us', `The '${key}' property must be a Boolean`]], lang);
  return true;
}

module.exports.checkTopCommand = checkTopCommand;

function checkQuery(query, lang) {
  if (_global.JS._.isUndefined(lang)) lang = _global.JE.i18nLang;
  if (_global.JS._.isUndefined(query)) return true;
  if (Object.prototype.toString.call(query) !== '[object Object]') _global.JS.throwError('ParamTypeError', null, null, [['zh-cn', `传入的 'query' 参数必须是 Object 类型`], ['en-us', `The 'query' param must be an Object`]], lang);
  if (!_global.JS._.isUndefined(query.where) && Object.prototype.toString.call(query.where) !== '[object Object]') _global.JS.throwError('PropTypeError', null, null, [['zh-cn', `属性 'where' 必须是 Object 类型`], ['en-us', `The 'whre' property must be an Object`]], lang);
  if (!_global.JS._.isUndefined(query.order) && !_global.JS._.isString(query.order) && !_global.JS._.isArray(query.order)) _global.JS.throwError('PropTypeError', null, null, [['zh-cn', `属性 'order' 必须是 String 或者 Array 类型`], ['en-us', `The 'order' property must be a String or an Array`]], lang);
  if (_global.JS._.isString(query.order) && query.order === '') _global.JS.throwError('PropEmptyError', null, null, [['zh-cn', `属性 'order' 不能为空字符串`], ['en-us', `The 'order' property can not be an empty String`]], lang);
  if (!_global.JS._.isUndefined(query.size) && !_global.JS._.isInteger(query.size)) _global.JS.throwError('PropTypeError', null, null, [['zh-cn', `属性 'size' 必须是整数`], ['en-us', `The 'size' property must be an integer`]], lang);
  if (query.size < 0) _global.JS.throwError('PropValueInvalidError', null, null, [['zh-cn', `属性 'size' 必须是正整数或 0`], ['en-us', `The 'size' property must be a positive integer or 0`]], lang);
  if (!_global.JS._.isUndefined(query.page) && !_global.JS._.isInteger(query.page)) _global.JS.throwError('PropTypeError', null, null, [['zh-cn', `属性 'page' 必须是整数`], ['en-us', `The 'page' property must be an integer`]], lang);
  if (query.page <= 0) _global.JS.throwError('PropValueInvalidError', null, null, [['zh-cn', `属性 'page' 必须是正整数`], ['en-us', `The 'page' property must be a positive integer`]], lang);
  if (!_global.JS._.isUndefined(query.page) && _global.JS._.isUndefined(query.size)) _global.JS.throwError('PropMissingError', null, null, [['zh-cn', `如果定义了 'page' 属性, 则 'size' 属性也是必需的`], ['en-us', `If you defined a 'page' property, then a 'size' property is required too`]], lang);
  return true;
}

module.exports.checkQuery = checkQuery;

function checkAfter(after, lang) {
  if (_global.JS._.isUndefined(lang)) lang = _global.JE.i18nLang;
  if (_global.JS._.isUndefined(after)) return true;
  if (Object.prototype.toString.call(after) !== '[object Object]' && !_global.JS._.isArray(after)) _global.JS.throwError('ParamTypeError', null, null, [['zh-cn', `属性 'after' 必须是 Object 或者 Array 类型`], ['en-us', `The property 'after' must be an Object or an Array`]], lang);
  return true;
}

module.exports.checkAfter = checkAfter;

function checkData(type, data, lang) {
  if (_global.JS._.isUndefined(lang)) lang = _global.JE.i18nLang;
  if (_global.JS._.isUndefined(data) && type !== 'delete') _global.JS.throwError('ParamMissingError', null, null, [['zh-cn', `对于 'create|update|increase|decrease' 操作，属性 'data' 必须要定义`], ['en-us', `Property 'data' is required for 'create|update|increase|decrease' type operations`]], lang);

  if (type === 'create') {
    if (Object.prototype.toString.call(data) !== '[object Object]' && !_global.JS._.isArray(data)) _global.JS.throwError('PropTypeError', null, null, [['zh-cn', `对于 'create' 操作，属性 'data' 必须是 Object 或者 Array 类型`], ['en-us', `Property 'data' must be an Object or an Array for create type operations`]], lang);
    if (Object.prototype.toString.call(data) === '[object Object]' && Object.keys(data).length === 0) _global.JS.throwError('PropEmptyError', null, null, [['zh-cn', `属性 'data' 不能为空对象`], ['en-us', `The 'data' Property can not be an empty Object`]], lang);
    if (_global.JS._.isArray(data) && data.length === 0) _global.JS.throwError('PropEmptyError', null, null, [['zh-cn', `属性 'data' 不能为空数组`], ['en-us', `The 'data' Property should be an Array which includes at least one element`]], lang);
  }

  if (type === 'update') {
    if (Object.prototype.toString.call(data) !== '[object Object]') _global.JS.throwError('PropTypeError', null, null, [['zh-cn', `对于 'update' 操作，属性 'data' 必须是 Object 类型`], ['en-us', `Property 'data' must be an Object for 'update' type operations`]], lang);
    if (Object.prototype.toString.call(data) === '[object Object]' && Object.keys(data).length === 0) _global.JS.throwError('PropEmptyError', null, null, [['zh-cn', `属性 'data' 不能为空对象`], ['en-us', `The 'data' Property can not be an empty Object`]], lang);
  }

  if (type === 'increase' || type === 'decrease') {
    if (Object.prototype.toString.call(data) !== '[object Object]' && !_global.JS._.isString(data)) _global.JS.throwError('PropTypeError', null, null, [['zh-cn', `对于 'increase|decrease' 操作，属性 'data' 必须是 String 或者 Object 类型`], ['en-us', `Property 'data' must be a String or an Object for 'increase|decrease' type operations`]], lang);
    if (Object.prototype.toString.call(data) === '[object Object]' && Object.keys(data).length === 0) _global.JS.throwError('PropEmptyError', null, null, [['zh-cn', `属性 'data' 不能为空对象`], ['en-us', `The 'data' Property can not be an empty Object`]], lang);
    if (_global.JS._.isString(data) && data === '') _global.JS.throwError('PropEmptyError', null, null, [['zh-cn', `属性 'data' 不能为空字符串`], ['en-us', `The 'data' Property can not be an empty String`]], lang);

    if (!_global.JS._.isString(data)) {
      let valid = true;
      let fails = [];

      _global.JS._.forEach(Object.keys(data), key => {
        if (isNaN(parseFloat(data[key]))) {
          fails.push(key);
          valid = false;
        }
      });

      if (valid === false) _global.JS.throwError('PropValueInvalidError', null, null, [['zh-cn', `字段 '${fails.join(',')}' 的值不是有效数字`], ['en-us', `Values of fields '${fails.join(',')}' are not numbers`]], lang);
    }
  }

  if (type === 'delete' && !_global.JS._.isUndefined(data)) _global.JS.throwError('PropSpilthError', null, null, [['zh-cn', `对于 'delete' 操作，属性 'data' 不应该出现`], ['en-us', `The 'data' Property should not exist in 'delete' type operations`]], lang);
  return true;
}

module.exports.checkData = checkData;

function safeString(str) {
  if (!_global.JS._.isString(str)) return str;
  return xss.process(str);
}

module.exports.safeString = safeString;

function hasSqlInjection(str) {
  if (!_global.JS._.isString(str)) return false;
  const sql = new RegExp("w*((%27)|('))((%6F)|o|(%4F))((%72)|r|(%52))", 'i');
  const sqlMeta = new RegExp("(%27)|(')|(--)|(%23)|(#)", 'i');
  const sqlMeta2 = new RegExp("(()|(=))[^\n]*((%27)|(')|(--)|(%3B)|(;))", 'i');
  const sqlUnion = new RegExp("((%27)|('))union", 'i');
  return sql.test(str) || sqlMeta.test(str) || sqlMeta2.test(str) || sqlUnion.test(str);
}

module.exports.hasSqlInjection = hasSqlInjection;